@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: mboxlist.c,v 1.236 2004/05/22 03:45:51 rjs3 Exp $
+ * $Id: mboxlist.c,v 1.240 2004/07/26 18:08:03 ken3 Exp $
  */
 
 #include <config.h>
@@ -134,7 +134,7 @@ char *mboxlist_makeentry(int mbtype, const char *part, const char *acl)
     return mboxent;
 }
 
-static const int get_deleteright(void)
+static int get_deleteright(void)
 {
     const char *r = config_getstring(IMAPOPT_DELETERIGHT);
 
@@ -384,7 +384,8 @@ mboxlist_mycreatemailboxcheck(char *name,
     if(!isadmin && force_user_create) return IMAP_PERMISSION_DENIED;
 
     /* User has admin rights over their own mailbox namespace */
-    if (mboxname_userownsmailbox(userid, name)) {
+    if (mboxname_userownsmailbox(userid, name) &&
+	(config_implicitrights & ACL_ADMIN)) {
 	isadmin = 1;
     }
 
@@ -914,10 +915,12 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
        user.<x> with no dots after it */
     if ((p = mboxname_isusermailbox(name, 1))) {
 	/* Can't DELETE INBOX (your own inbox) */
-	int len = config_virtdomains ? strcspn(userid, "@") : strlen(userid);
-	if (userid && (len == strlen(p)) && !strncmp(p, userid, len)) {
-	    r = IMAP_MAILBOX_NOTSUPPORTED;
-	    goto done;
+	if (userid) {
+	    int len = config_virtdomains ? strcspn(userid, "@") : strlen(userid);
+	    if ((len == strlen(p)) && !strncmp(p, userid, len)) {
+		r = IMAP_MAILBOX_NOTSUPPORTED;
+		goto done;
+	    }
 	}
 
 	/* Only admins may delete user */
@@ -1897,10 +1900,23 @@ int mboxlist_findall(struct namespace *namespace __attribute__((unused)),
     char *pat = NULL;
 
     if (config_virtdomains) {
-	if (userid && (p = strrchr(userid, '@'))) {
-	    userlen = p - userid;
-	    domainlen = strlen(p); /* includes separator */
-	    snprintf(domainpat, sizeof(domainpat), "%s!%s", p+1, pattern);
+	char *domain;
+
+	if (userid && (domain = strrchr(userid, '@'))) {
+	    userlen = domain - userid;
+	    domainlen = strlen(domain); /* includes separator */
+
+	    if ((p = strchr(pattern , '!'))) {
+		if ((p-pattern != domainlen-1) ||
+		    strncmp(pattern, domain+1, domainlen-1)) {
+		    /* don't allow cross-domain access */
+		    return IMAP_MAILBOX_BADNAME;
+		}
+
+		pattern = p+1;
+	    }
+
+	    snprintf(domainpat, sizeof(domainpat), "%s!%s", domain+1, pattern);
 	}
 	if ((p = strrchr(pattern, '@'))) {
 	    /* global admin specified mbox@domain */
