@@ -1,6 +1,6 @@
 /* imclient.c -- Streaming IMxP client library
  *
- * $Id: imclient.c,v 1.86 2005/10/28 14:45:22 ken3 Exp $
+ * $Id: imclient.c,v 1.84 2003/10/22 18:50:12 rjs3 Exp $
  *
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
@@ -1570,7 +1570,8 @@ static char issuer_CN[CCERT_BUFSIZ];
 static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 {
     if (cert_file != NULL) {
-	if (SSL_CTX_use_certificate_chain_file(ctx, cert_file) <= 0) {
+	if (SSL_CTX_use_certificate_file(ctx, cert_file,
+					 SSL_FILETYPE_PEM) <= 0) {
 	  printf("[ unable to get certificate from '%s' ]\n", cert_file);
 	  return (0);
 	}
@@ -1669,6 +1670,8 @@ static int tls_rand_init(void)
 #endif
 }
 
+char *var_tls_CAfile="";
+char *var_tls_CApath="";
  /*
   * This is the setup routine for the SSL client. 
   *
@@ -1676,10 +1679,7 @@ static int tls_rand_init(void)
   */
 
 static int tls_init_clientengine(struct imclient *imclient,
-				 int verifydepth, char *var_tls_cert_file,
-                                 char *var_tls_key_file,
-                                 char *var_tls_CAfile,
-                                 char *var_tls_CApath)
+				 int verifydepth, char *var_tls_cert_file, char *var_tls_key_file)
 {
     int     off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
@@ -1707,11 +1707,11 @@ static int tls_init_clientengine(struct imclient *imclient,
     
     /* debugging   SSL_CTX_set_info_callback(imclient->tls_ctx, apps_ssl_info_callback); */
 
-    if (var_tls_CAfile == NULL || strlen(var_tls_CAfile) == 0)
+    if (strlen(var_tls_CAfile) == 0)
 	CAfile = NULL;
     else
 	CAfile = var_tls_CAfile;
-    if (var_tls_CApath == NULL || strlen(var_tls_CApath) == 0)
+    if (strlen(var_tls_CApath) == 0)
 	CApath = NULL;
     else
 	CApath = var_tls_CApath;
@@ -1722,11 +1722,11 @@ static int tls_init_clientengine(struct imclient *imclient,
 	    printf("[ TLS engine: cannot load CA data ]\n");
 	    return -1;
 	}
-    if (var_tls_cert_file == NULL || strlen(var_tls_cert_file) == 0)
+    if (strlen(var_tls_cert_file) == 0)
 	c_cert_file = NULL;
     else
 	c_cert_file = var_tls_cert_file;
-    if (var_tls_key_file == NULL || strlen(var_tls_key_file) == 0)
+    if (strlen(var_tls_key_file) == 0)
 	c_key_file = NULL;
     else
 	c_key_file = var_tls_key_file;
@@ -1973,23 +1973,13 @@ int tls_start_clienttls(struct imclient *imclient,
 	   tls_cipher_usebits, tls_cipher_algbits);*/
     return 0;
 }
-#endif /* HAVE_SSL */
-
-int imclient_havetls () {
-#ifdef HAVE_SSL
-  return 1;
-#else
-  return 0;
-#endif
-}
 
 int imclient_starttls(struct imclient *imclient,
-			     char *cert_file,
-			     char *key_file,
-                             char *CAfile,
-                             char *CApath)
+			     int verifydepth __attribute__((unused)),
+			     char *var_tls_cert_file, 
+			     char *var_tls_key_file,
+			     int *layer __attribute__((unused)))
 {
-#ifdef HAVE_SSL
   int result;
   struct authresult theresult;
   unsigned ssf;
@@ -2004,8 +1994,7 @@ int imclient_starttls(struct imclient *imclient,
     imclient_processoneevent(imclient);
   }
 
-  result=tls_init_clientengine(imclient, 10, cert_file, key_file,
-                               CAfile, CApath);
+  result=tls_init_clientengine(imclient, 10, var_tls_cert_file, var_tls_key_file);
   if (result!=0)
   {
     printf("[ TLS engine failed ]\n");
@@ -2041,8 +2030,5 @@ int imclient_starttls(struct imclient *imclient,
   if (result!=SASL_OK) return 1;
 
   return 0;
-#else
-  printf("[ TLS support not present (imclient_starttls) ]\n");
-  return 1;
-#endif /* HAVE_SSL */
 }
+#endif /* HAVE_SSL */
